from .connection import ConnectionSsdby
from loguru import logger
from valkyt.utils import Stream, File

class Ssdby(ConnectionSsdby):
    def __init__(self, host, port, **kwargs):
        super().__init__(host, port, **kwargs)
        
    def already_in_ssdb(self, tube: str, id: str, only_check: bool = False) -> bool:
        exist = self.client.hexists("{}".format(tube), "{}".format(id))
        if not exist:
            if not only_check:
                logger.info(f'[+] NEW ITEM ADD TO SSDB :: ITEM [ {id} ]')
                hset = self.client.hset(
                    "{}".format(tube), "{}".format(id), "{}".format(1)
                )
                if hset:
                    return False
            else:
                logger.info(f"ONLY CHECK ITEM NOT FOUND IN SSDB")
                return False
        else:
            logger.info(f'ITEM ALREDY IN SSDB :: ITEM [ {id} ]')
            return True
        ...
        
    def close(self) -> None:
        self.client.close()


import logging
from click import style

logging.basicConfig(datefmt='%m/%d/%Y %I:%M:%S %p', encoding="utf-8", level=logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')

console = logging.StreamHandler()
console.setLevel(level=logging.DEBUG) 
console.setFormatter(formatter)

logger = logging.getLogger()
for existing_handler in logger.handlers[:]:
       logger.removeHandler(existing_handler)

logger.addHandler(console)

class Stream:

       @staticmethod
       def info(name: str, total: int, success: int, error: int) -> None:
              logger.info(f'[ {style(name, fg="bright_green")} ] :: {style("total data", fg="magenta")}: [ {total} ] | {style("success", fg="bright_blue")}: [ {success} ] | {style("error", fg="red")}: [ {error} ]')

       @staticmethod
       def cards(name: str, text: str, page: int, total: int) -> None:
              logger.info(f'[ {style(name, fg="bright_green")} ] :: {style(f"total {text}", fg="magenta")}: [ {total} ] | {style("page", fg="magenta")}: [ {page} ]')

       @staticmethod
       def s3(bucket: str, response: int) -> None:
              logger.info(f'[ {style("UPLOAD TO S3", fg="bright_green")} ] :: {style(f"bucket", fg="magenta")}: [ {bucket} ] | {style("response", fg="magenta")}: [ {response} ]')

       @staticmethod
       def end(start: float, end: float) -> float:
              logger.info(f'[ {style("SUCCESS", fg="red")} ] :: {style(f"time", fg="magenta")}: [ {end - start} ]')

       @staticmethod
       def sql_domain(domain: str, path: str) -> None:
              logger.info(f'[ {style("SQL", fg="bright_green")} ] :: {style(f"NEW DOMAIN", fg="bright_blue")}: [ {domain} ] | {style(f"PATH", fg="magenta")}: [ {path} ]')
              ...

       @staticmethod
       def share(path: str) -> None:
              logger.info(f'[ {style("SHARE", fg="bright_green")} ] :: {style(f"SOURCE", fg="bright_blue")}: [ {path} ]')
              logger.info(f'[ {style("SHARE", fg="bright_green")} ] :: {style(f"DESTINATION", fg="bright_blue")}: [ {path} ]')
              ...

       @staticmethod
       def shareKafka(path: str) -> None:
              logger.info(f'[ {style("SHARE", fg="bright_green")} ] :: {style(f"SOURCE", fg="bright_blue")}: [ {path} ]')
              ...

       @staticmethod
       def found(process: str, message: int, total: int) -> None:
              logger.info(f'[ {style(process, fg="bright_green")} ] :: {style(f"{message}", fg="bright_blue")}: [ {total} ]')
              ...
              
       @staticmethod
       def zip_stream(process: str, source: str, destination: str, password: str) -> None:
              logger.info(f'[ {style(process, fg="bright_green")} ] :: {style("SOURCE", fg="bright_blue")}: [ {source} ] | {style("DESTINATION", fg="bright_blue")}: [ {destination} ] | {style("PASSWORD", fg="bright_blue")}: [ {password} ]')
              
       @staticmethod
       def one(process: str, message: str, value: str) -> None:
              logger.info(f'[ {style(process, fg="bright_green")} ] :: {style(message, fg="bright_blue")}: [ {value} ]')
              
       @staticmethod
       def errone(process: str, message: str, value: str) -> None:
              logger.error(f'[ {style(process, fg="bright_green")} ] :: {style(message, fg="bright_blue")}: [ {value} ]')


import os
import json

from json import dumps

from typing import Any
from botocore.exceptions import ClientError
from .connection import ConnectionS3

from valkyt.utils import Stream, File

class S3(ConnectionS3):
    def __init__(self, bucket, service_name, aws_access_key_id, aws_secret_access_key, endpoint_url, **kwargs):
        super().__init__(bucket, service_name, aws_access_key_id, aws_secret_access_key, endpoint_url, **kwargs)
        
    def read_json(self, key: str) -> Any:
        try:
            response = self.client.get_object(
                Bucket=self.bucket,
                Key=key
            )
            content = response['Body'].read().decode('utf-8')
            return json.loads(content)
        except ClientError as e:
            if e.response['Error']['Code'] == 'NoSuchKey':
                empty_json = {}
                self.client.put_object(
                    Bucket=self.bucket,
                    Key=key,
                    Body=json.dumps(empty_json, indent=2, ensure_ascii=False)
                )
                return empty_json
            else:
                raise 

    def upload_json(self, destination: str, body: dict, send: bool = True) -> int:
        if send: 
            response: dict = self.client.put_object(
                Bucket=self.bucket, 
                Key=destination, 
                Body=dumps(body, indent=2, ensure_ascii=False)
                )
            Stream.s3(destination, response['ResponseMetadata']['HTTPStatusCode'])
            return response['ResponseMetadata']['HTTPStatusCode']
        ...
        
    def upload_file(self, path: str, destination: str, send: bool = True) -> int:
        if send: 
            response: dict = self.client.put_object(
                                Bucket=self.bucket,
                                Key = destination, 
                                Body = open(path, 'rb')
                            )
            Stream.s3(destination, response['ResponseMetadata']['HTTPStatusCode'])
            return response['ResponseMetadata']['HTTPStatusCode']
        ...
        
    def upload(self, body: any, destination: str, send: bool = True) -> int:
        if send: 
            response: dict = self.client.put_object(
                                Bucket=self.bucket,
                                Key = destination, 
                                Body = body
                            )
            Stream.s3(destination, response['ResponseMetadata']['HTTPStatusCode'])
            return response['ResponseMetadata']['HTTPStatusCode']
        ...
        
    def local2s3(self, source: str) -> None:
        for root, dirs, files in os.walk(source.replace('\\', '/')):
            for file in files:
                file_path = os.path.join(root, file).replace('\\', '/')
                
                if 'json' in file_path:
                    _temp: dict = File.read_json(file_path)
                    _temp["path_data_raw"] = list(
                        map(
                            lambda x: x.replace('//', '/'),
                            _temp["path_data_raw"]
                        )
                    )
                    
                    File.write_json(
                        file_path,
                        _temp
                    )
                
                Stream.share(file_path)

                S3.upload_file(
                    path=file_path,
                    destination=file_path,
                )



import os
from redis import Redis
from loguru import logger

class ConnectionRedys:
    def __init__(self, host: str, port: int, db: int, **kwargs) -> None:
        logger.info(f"START CREATE REDIS CONNECTIONS :: HOST [ {host} ] | PORT [ {port} ] | DB [ {db} ]")
        self.client: Redis = Redis(
            host=host,
            port=port,
            db=db,
            decode_responses=True,
            **kwargs
        )
        ...




import json
import pika

from loguru import logger
from .connection import RabbitMQConnection

class Rabbyt(RabbitMQConnection):
    def send(self, data: dict):
        channel = self.connect()

        channel.exchange_declare(
            exchange=self.exchange, 
            exchange_type='direct', 
            durable=True
        )
        channel.queue_declare(queue=self.queue, durable=True)
        channel.queue_bind(
            exchange=self.exchange,
            queue=self.queue,
            routing_key=self.routing_key
        )

        body = json.dumps(data)
        channel.basic_publish(
            exchange=self.exchange,
            routing_key=self.routing_key,
            body=body,
            properties=pika.BasicProperties(delivery_mode=2)
        )

        logger.info(f"Message send to RabbitMQ: {body}")


import datetime as date
import pytz
from tqdm import tqdm

from dateutil.relativedelta import relativedelta

from time import strftime, time, localtime, struct_time, mktime, sleep
from random import randint
from datetime import datetime, timezone, timedelta

class Time:
    
    @staticmethod
    def timer(self, message: str = "SLEEP BEFORE NEXT PROCESS", start: str = 100, end: int = 300):
        w: int = randint(start, end)
        for i in tqdm(
            range(w*10),
            desc=message,
            bar_format='[{elapsed}<{remaining}] {n_fmt}/{total_fmt} | {l_bar}{bar} {rate_fmt}{postfix}', 
            colour='green', 
            leave=False
        ):
            sleep(0.1)
        ...
    
    @staticmethod
    def current_month_year():
        now = datetime.now()
        return f"{now.month}{now.year}"

    @staticmethod
    def convert_time(times: str) -> int:
        dt = date.datetime.fromisoformat(times)
        dt = dt.replace(tzinfo=timezone.utc) 
        return int(dt.timestamp())
        ...

    @staticmethod
    def now():
        return strftime('%Y-%m-%d %H:%M:%S')
        ...

    @staticmethod
    def today():
        return datetime.now().strftime("%Y-%m-%d")
        ...
    @staticmethod
    def yesterday():
        jakarta_tz = pytz.timezone('Asia/Jakarta')
        now_in_jakarta = datetime.now(jakarta_tz)
        one_day_before = now_in_jakarta - timedelta(days=1)
        return one_day_before.strftime("%Y-%m-%d")
        ...
    
    @staticmethod
    def change_format(dates: str) -> str:
        try: return dates.replace('T', ' ')
        except Exception: return dates
        ...

    @staticmethod
    def epoch():
        return int(time())
        ...

    @staticmethod
    def epoch_ms():
        return str(round(time() * 1000))
        ...

    @staticmethod        
    def current_year():
        return datetime.now().year

    @staticmethod
    def epoch_today():
        current_time = time()
        local_time = localtime(current_time)
        midnight_time = struct_time((local_time.tm_year, local_time.tm_mon, local_time.tm_mday, 0, 0, 0, local_time.tm_wday, local_time.tm_yday, local_time.tm_isdst))
        midnight_epoch = mktime(midnight_time)
        return int(midnight_epoch)


    @staticmethod
    def relative2date(relative_time: str) -> str:
        if relative_time.startswith('se'):
            relative_time = f'1 {relative_time[2:]}'
            
        parsed_time = datetime.now()

        if 'menit' in relative_time:
            minutes_ago = int(relative_time.split()[0])
            parsed_time -= relativedelta(minutes=minutes_ago)

        elif 'jam' in relative_time:
            hours_ago = int(relative_time.split()[0])
            parsed_time -= relativedelta(hours=hours_ago)

        elif 'hari' in relative_time:
            days_ago = int(relative_time.split()[0])
            parsed_time -= relativedelta(days=days_ago)

        elif 'minggu' in relative_time:
            weeks_ago = int(relative_time.split()[0])
            parsed_time -= relativedelta(weeks=weeks_ago)

        elif 'bulan' in relative_time:
            months_ago = int(relative_time.split()[0])
            parsed_time -= relativedelta(months=months_ago)

        elif 'tahun' in relative_time:
            years_ago = int(relative_time.split()[0])
            parsed_time -= relativedelta(years=years_ago)

        return parsed_time.strftime('%Y-%m-%d %H:%M:%S')
        ...


import os
import requests
import tempfile
import shutil
import yt_dlp

from uuid import uuid4
from dekimashita import Dekimashita
from requests import Response
from loguru import logger

from .directory import Dir
from .fileIO import File
from playwright.sync_api import Page, ElementHandle
from playwright.async_api import Page, ElementHandle

class Down:

    @staticmethod
    def curl(url: str, path: str, headers: dict = None, cookies: dict = None, extension: str = None) -> Response:
        Dir.create_dir(paths='/'.join(path.split('/')[:-1]))
        response = requests.get(url=url, headers=headers, cookies=cookies, verify=False, timeout=900)
        with open(path, 'wb') as f:
            f.write(response.content)

        return response
    
    @staticmethod
    def curlv2(path: str, response: Response, extension: str = None) -> Response:
        Dir.create_dir(paths='/'.join(path.split('/')[:-1]))
        with open(path, 'wb') as f:
            f.write(response.content)
            
    # @staticmethod
    # def playwright(page: Page, loc: ElementHandle, base_desctination: str, s3: bool, save: bool) -> str:
    #     with page.expect_download() as download_info:
    #         loc.click()
    #         download = download_info.value
        
    #     temp_dir: str = tempfile.mkdtemp().replace('\\', '/')
    #     filename: str = download.suggested_filename
    #     temp_path = temp_dir+'/'+filename
        
    #     logger.info(f'DOWNLOAD FILE [ {filename} ]')
    #     download.save_as(temp_path)
        
    #     extention: str = filename.split('.')[-1]
    #     destination_path: str = base_desctination + extention + '/' + filename
    #     S3.upload(
    #         body=File.read_byte(temp_path),
    #         destination=destination_path,
    #         send=s3
    #     )
    #     if save:
    #         File.write_byte(
    #             path=destination_path,
    #             media=File.read_byte(temp_path),
    #         )
    #     shutil.rmtree(temp_dir)
    #     return destination_path

    # @staticmethod
    # async def asyncPlaywright(page: Page, loc: ElementHandle, base_desctination: str, s3: bool, save: bool) -> str:
    #     async with page.expect_download() as download_info:
    #         await loc.click()
    #     download = await download_info.value
    #     temp_dir = tempfile.mkdtemp().replace('\\', '/')
    #     filename = download.suggested_filename
        
    #     temp_path = os.path.join(temp_dir, filename)
    #     logger.info(f'DOWNLOAD FILE [ {filename} ]')
        
    #     await download.save_as(temp_path)
        
    #     extention: str = filename.split('.')[-1]
    #     destination_path: str = base_desctination + extention + '/' + filename
    #     S3.upload(
    #         body=File.read_byte(temp_path),
    #         destination=File.vdir(destination_path),
    #         send=s3
    #     )
    #     if save:
    #         File.write_byte(
    #             path=File.vdir(destination_path),
    #             media=File.read_byte(temp_path),
    #         )
    #     shutil.rmtree(temp_dir)
    #     return File.vdir(destination_path)
    
    # @staticmethod
    # def syncPlaywright(page: Page, loc: ElementHandle, base_desctination: str, s3: bool, save: bool) -> str:
    #     with page.expect_download() as download_info:
    #         loc.click()
    #     download = download_info.value
    #     temp_dir = tempfile.mkdtemp().replace('\\', '/')
    #     filename = download.suggested_filename
        
    #     temp_path = os.path.join(temp_dir, filename)
        
    #     download.save_as(temp_path)
        
    #     extention: str = filename.split('.')[-1]
    #     name: str = filename.split('.')[0]
    #     destination_path: str = base_desctination + extention + '/' + Dekimashita.vdir(name.replace('-', '_')) + '.' + extention
    #     if os.path.exists(destination_path):
    #         destination_path: str = base_desctination + extention + '/' + Dekimashita.vdir(name.replace('-', '_')) + str(uuid4()).replace('-', '_') + '.' + extention
            
    #     logger.info(f'DOWNLOAD FILE [ {destination_path.split("/")[-1]} ]')
    #     S3.upload(
    #         body=File.read_byte(temp_path),
    #         destination=File.vdir(destination_path),
    #         send=s3
    #     )
    #     if save:
    #         File.write_byte(
    #             path=File.vdir(destination_path),
    #             media=File.read_byte(temp_path),
    #         )
    #     shutil.rmtree(temp_dir)
    #     return File.vdir(destination_path)
    
    @staticmethod
    def youtube_download(url):
        logger.info(f'VIDEO DOWNLOAD :: URL [ {url} ]')
        ydl_opts = {
            'format': 'bestvideo[ext=mp4]+bestaudio[ext=m4a]/best[ext=mp4]/best',
            'http_headers': {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'},
            'nocheckcertificate': True
        }
        
        try:
            with yt_dlp.YoutubeDL(ydl_opts) as ydl:
                print("start...")
                
                info = ydl.extract_info(url, download=False)
                
                if 'entries' in info:
                    video_info = info['entries'][0]
                else:
                    video_info = info
                
                formats = video_info.get('formats', [])
                
                def safe_get_filesize(f):
                    return f.get('filesize') or f.get('filesize_approx') or 0
                
                best_format = max(formats, key=safe_get_filesize)
                video_url = best_format['url']
                
                with ydl.urlopen(video_url) as response:
                    video_bytes = response.read()
                
            print("Unduhan selesai!")
            return video_bytes
        except Exception as e:
            print(f"Terjadi kesalahan: {str(e)}")
            return None
        
    



import os


class Dir:

    @staticmethod
    def create_dir(paths: str, create: bool = True) -> str:
        try: 
            if create: os.makedirs(paths)
        except Exception as err: ...
        finally: return paths
        ...

    @staticmethod
    def basedir(path: str) -> str:
        return os.path.dirname(path)
        ...


import json
import os
from loguru import logger
from typing import List
from .directory import Dir


class File:

    @staticmethod
    def write_json(path: str, content: any) -> None:
        try:
            Dir.create_dir(Dir.basedir(path))
            with open(path, 'w', encoding= "utf-8") as file:
                json.dump(content, file, ensure_ascii=False, indent=2, default=str)

        except Exception as err:
            Dir.create_dir(Dir.basedir(path))
            with open(path, 'w') as file:
                json.dump(content, file, indent=2, default=str)
        ...

    @staticmethod
    def write_str(path: str, content: any) -> None:
        Dir.create_dir(Dir.basedir(path))
        with open(path, 'w', encoding="utf-8") as file:
            file.writelines(content)
        ...

    @staticmethod
    def write(path: str, content: any) -> None:
        Dir.create_dir(Dir.basedir(path))
        with open(path, 'a', encoding="utf-8") as file:
            file.write(content+'\n')
        ...

    @staticmethod
    def write_byte(path: str, media: any) -> None:
        Dir.create_dir(Dir.basedir(path))
        with open(path, 'wb') as file:
            file.write(media)
        ...

    @staticmethod
    def read_json(path: str) -> any:
        Dir.create_dir(Dir.basedir(path))
        try:
            with open(path, 'r', encoding='utf-8') as file:
                data = json.load(file)
            return data
        except Exception as err:
            return {}
        ...
        
    @staticmethod
    def read(path: str) -> any:
        with open(path, 'r', encoding='utf-8') as file:
            html_content = file.read()
            
        return html_content
    def read_byte(path: str) -> any:
        with open(path, "rb") as file:
            download_bytes = file.read()
            
        return download_bytes

    @staticmethod
    def read_list_json(path: str):
        Dir.create_dir(Dir.basedir(path))
        if not os.path.exists(path): File.write_json(path, [])
        with open(path, encoding="utf-8") as f:
            data = f.read()
            if not data: data = '[]'
        return json.loads(data)
        ...
    
    @staticmethod
    def list_dir(path: str) -> List[str]:
        return os.listdir(path)
        ...

    @staticmethod
    def get_format(path: str) -> str:
        _, extension = os.path.splitext(path)
        return extension.replace('.', '')
        ...

    @staticmethod
    def name_file(path: str) -> str:
        return os.path.basename(path)
        ...
        
    @staticmethod
    def vdir(text: str) -> str:
        try:
            return text.lower().replace(' ', '_')
        except Exception as err:
            logger.error(f'VDIR ERROR :: MESSAGE [ {str(err)} ]')
            return text
        ...

    @staticmethod
    def check_file(path: str) -> bool:
        
        if not os.path.exists(path):
            return False
        
        if os.path.getsize(path) == 0:
            return False
        
        return True



import atexit
from greenstalk import Client, TimedOutError, Job

class ConnectionBeanstalk:
    def __init__(self, host: str, port: str, tube: str) -> None:
        self.client = Client((host, port), use=tube, watch=tube)
        atexit.register(self.close)
        ...
        
    def close(self) -> None:
        self.client.close()
        ...


from urllib.parse import urlparse
class UrlPars:

    @staticmethod
    def get_domain(url: str) -> str | None:
        try:
            parsed_url = urlparse(url)
            domain = parsed_url.netloc
            return domain
        except Exception as e:
            print(f"error : {e}")
            return None


import redis
from loguru import logger

class ConnectionSsdby:
    def __init__(self, host: str, port: int, **kwargs) -> None:
        logger.info(f"START CREATE SSDB CONNECTIONS :: HOST [ {host} ] | PORT [ {port} ]")
        self.client = redis.StrictRedis(
            host=host,
            port=port,
            **kwargs
        )



import pika
import json
from loguru import logger

class RabbitMQConnection:
    def __init__(
        self,
        username: str,
        password: str,
        host: str,
        port: str,
        virtual_host: str,
        queue: str,
        routing_key: str,
        exchange: str
    ):
        self.queue: str = queue
        self.routing_key: str = routing_key
        self.exchange: str = exchange
        
        credentials = pika.PlainCredentials(
            username,
            password
        )
        parameters = pika.ConnectionParameters(
            host=host,
            port=port,
            virtual_host=virtual_host,
            credentials=credentials
        )
        self.connection = pika.BlockingConnection(parameters)
        self.channel = self.connection.channel()
        return self.channel

    def close(self):
        if self.connection:
            self.connection.close()


import unittest

class Unytest:
    def __init__(self):
        self.unytestCase = unittest.TestCase()
        self.asyncUnytestCase = unittest.IsolatedAsyncioTestCase()
        ...
        
    def __view(
        self,
        assert_func,
        *args,
        message: str = "",
        context: str = "",
        bot = None,
        **kwargs
    ) -> None:
        try:
            assert_func(*args)
            self.__log.debug(f"[ PASS ] {message} :: {context}")
        except AssertionError as e:
            self.__log.error(f"[ FAIL ] {message} :: {context} -- {e}")
            if bot:
                bot(**kwargs)


import json

from typing import Generator, Dict
from loguru import logger

from .connection import ConnectionBeanstalk, TimedOutError, Client, Job

class Beanstalk:
    def __init__(self, host: str, port: str, tube: str) -> None:
        self.connection: ConnectionBeanstalk = ConnectionBeanstalk(host, port, tube)
        self.job: Job = None
        ...
        
    def get(self) -> Generator[Dict[str, any], any, None]:
        while True:
            try:
                self.job = self.connection.client.reserve(timeout=10)
                yield json.loads(self.job.body)
                self.__delete()
            except TimedOutError:
                yield None
            except BrokenPipeError:
                raise
            except Exception:
                raise
            
    def put(self, output: str, **kwargs) -> None:
        self.connection.client.put(body=output, **kwargs)
        ...

    def __delete(self):
        if self.job:
            try:
                self.connection.client.delete(self.job)
                self.job = None
            except Exception as err:
                logger.warning(f'BEANSTALK DELETED :: MESSAGE [ {str(err)} ]')

    def __bury(self, **kwargs) -> None:
        if self.job:
            try:
                self.connection.client.bury(self.job, **kwargs)
                self.job = None
            except Exception as err:
                logger.warning(f'BEANSTALK BURRY :: MESSAGE [ {str(err)} ]')
        ...

    def exception_handler(self, **kwargs) -> None:
        try:
            action = kwargs.get("action")
            if action == "delete":
                self.__delete()
            else:
                self.__bury()
        except Exception as err:
            logger.error(f"EXCEPTION HANDLER ERROR :: MESSAGE [ {str(err)} ]")



from typing import List, Dict, Any

def split_list(datas: List[Any], length_each: int) -> List[List[Any]]:
    """
    Splits a list into sublists of specified length.
    
    Takes a list and divides it into multiple sublists where each sublist
    (except possibly the last one) has the specified length.
    
    Args:
        datas (List[Any]): The input list to be split
        length_each (int): The maximum length of each sublist
    
    Returns:
        List[List[Any]]: A list containing the sublists
        
    Examples:
        >>> split_list([1, 2, 3, 4, 5], 2)
        [[1, 2], [3, 4], [5]]
    """
    results: List[Any] = []
    for i in range(0, len(datas), length_each):
        results.append(datas[i:i+length_each])
    return results

def to_float(text: str) -> float | str:
    """
    Safely converts a string to float.
    
    Attempts to convert a string to float. Returns the original string
    if conversion fails.
    
    Args:
        text (str): The string to convert
    
    Returns:
        float | str: The converted float or original string if conversion fails
        
    Examples:
        >>> to_float("3.14")
        3.14
        >>> to_float("abc")
        "abc"
    """
    try:
        return float(text)
    except Exception:
        return text

def to_int(text: str) -> int | str:
    """
    Safely converts a string to integer.
    
    Attempts to convert a string to integer. Returns the original string
    if conversion fails.
    
    Args:
        text (str): The string to convert
    
    Returns:
        int | str: The converted integer or original string if conversion fails
        
    Examples:
        >>> to_int("42")
        42
        >>> to_int("4.2")
        "4.2"
    """
    try:
        return int(text)
    except Exception:
        return text

def val(text: Any) -> Any:
    """
    Returns the input value if truthy, otherwise returns None.
    
    A helper function that returns None for falsy values (empty strings,
    empty lists, etc.) and the original value for truthy values.
    
    Args:
        text (Any): The value to evaluate
    
    Returns:
        Any: The original value if truthy, None otherwise
        
    Examples:
        >>> val("text")
        "text"
        >>> val("")
        None
    """
    if text: return text
    else: return None

def vlist_dict(data_list: List[Dict], key: str) -> List[Dict]:
    """
    Removes duplicate dictionaries from a list based on a key.
    
    Takes a list of dictionaries and returns a new list without duplicates,
    where uniqueness is determined by the specified key.
    
    Args:
        data_list (List[Dict]): List of dictionaries to process
        key (str): Dictionary key to check for duplicates
    
    Returns:
        List[Dict]: List with duplicate dictionaries removed
        
    Examples:
        >>> data = [{'id': 1, 'name': 'A'}, {'id': 1, 'name': 'B'}]
        >>> vlist_dict(data, 'id')
        [{'id': 1, 'name': 'A'}]
    """
    seen = set()
    result = []
    for d in data_list:
        if d[key] not in seen:
            seen.add(d[key])
            result.append(d)
    return result

def search_key_v2(data: Dict[str, Any], key: str, required_key: str = None) -> Any:
    """
    Recursively searches for a key in nested dictionaries (enhanced version).
    
    Searches through nested dictionaries to find a specified key. Can optionally
    return a value from a sub-dictionary if the key is found.
    
    Args:
        data (Dict[str, Any]): The dictionary to search
        key (str): The key to search for
        required_key (str, optional): If provided and key is found,
            returns data[key][required_key] if exists
    
    Returns:
        Any: The found value or None if not found
        
    Examples:
        >>> data = {'a': {'b': {'c': 1}}}
        >>> search_key_v2(data, 'b')
        {'c': 1}
        >>> search_key_v2(data, 'b', 'c')
        1
    """
    if key in data:
        if isinstance(data[key], dict):
            if required_key is None:
                return data[key]
            elif required_key in data[key]:
                return data[key][required_key]
        else:
            return None

    for k, v in data.items():
        if isinstance(v, dict):
            result = search_key_v2(v, key, required_key)
            if result is not None:
                return result
    return None

def search_key(data: Dict[str, Any], key: str) -> Any:
    """
    Recursively searches for a key in nested dictionaries.
    
    Searches through nested dictionaries to find a specified key.
    Returns the first occurrence found in a depth-first search.
    
    Args:
        data (Dict[str, Any]): The dictionary to search
        key (str): The key to search for
    
    Returns:
        Any: The found value or None if not found
        
    Examples:
        >>> data = {'a': 1, 'b': {'c': 2}}
        >>> search_key(data, 'c')
        2
    """
    if not isinstance(data, dict):
        return None
    if key in data:
        return data[key]
    for value in data.values():
        if isinstance(value, dict):
            result = search_key(value, key)
            if result is not None:
                return result
    return None


import os
import json

from json import dumps

from typing import Any, Dict
from loguru import logger
from botocore.exceptions import ClientError
from .connection import ConnectionRedys

class Redys(ConnectionRedys):
    def __init__(self, host, port, db, **kwargs):
        super().__init__(host, port, db, **kwargs)
        
    def check(self, key: str, id: str, **kwargs) -> Dict:
        """
        Examples:
            >>> check(key="xxxx:xxxx:xxxx", id="xxxxxx")
            {
                "data": xxxxx
            }
            
            >>> check(key="xxxx:xxxx:xxxx", id="xxxxxx")
            None
        """
        __key: str = "{}:{}".format(
            key,
            id
        )
        __key = self.client.get(__key)
        if __key:
            logger.info(f'DATA ALREDY IN REDIS :: ID [ {id} ]')
            return json.loads(__key)
        return None
        ...
        
    def push(self, data: str, key: str, id: str, **kwargs):
        """
        Examples:
            >>> push(
                data={
                    "data": xxxxx
                },
                key="xxxx:xxxx:xxxx",
                id="xxxxxx"
            )
        """
        self.client.set(
            "{}:{}".format(
                key,
                id
            ),
            json.dumps(
                data,
                ensure_ascii=False
            ),
            **kwargs
        )
        logger.info(f"NEW DATA ADD IN REDIS :: ID [ {id} ]")


import os
import json

from kafka import KafkaProducer

class ConnectionKafka:
    def __init__(self, bootstraps: list) -> KafkaProducer:
        self.client = KafkaProducer(bootstrap_servers=bootstraps)
        ...



from bs4 import BeautifulSoup
from pyquery import PyQuery as pq


class HtmlParser:
    def __init__(self):
        pass

    def bs4_parser(self, html, selector):
        result = None
        try:
            html = BeautifulSoup(html, "lxml")
            result = html.select(selector)
        except Exception as e:
            print(e)
        finally:
            return result

    def pyq_parser(self, html, selector):
        result = None
        try:
            html = pq(html)
            result = html(selector)
        except Exception as e:
            print(e)
        finally:
            return result



class Casting:

    @staticmethod
    def to_float(text: str) -> float | str:
        try:
            return float(text)
        except Exception:
            return text
        

    @staticmethod
    def to_int(text: str) -> float | str:
        try:
            return int(text)
        except Exception:
            return text
        
    @staticmethod
    def to_pascal_case(text: str, **kwargs):
        
        if kwargs.get('exemption') and text in kwargs.get('exemption'):
            return text
        if kwargs.get('lower') and text.upper() in kwargs.get('lower'):
            return text.lower()
        
        words = text.split()
        pascal_case_words = [word.capitalize() for word in words]
        return ' '.join(pascal_case_words)


import boto3

class ConnectionS3:
    def __init__(
        self, 
        bucket: str, 
        service_name: str, 
        aws_access_key_id: str, 
        aws_secret_access_key: str, 
        endpoint_url: str, 
        **kwargs
    ) -> None:
        self.bucket = bucket

        self.client = boto3.client(
            service_name=service_name,
            aws_access_key_id=aws_access_key_id, 
            aws_secret_access_key=aws_secret_access_key, 
            endpoint_url=endpoint_url
        )



from pyquery import PyQuery
from bs4 import BeautifulSoup

class Parser:
    def __init__(self) -> None:
        pass

    def ex(self, html: PyQuery, selector: str) -> PyQuery:
        result = None
        try:
            html: str = PyQuery(html)
            result = html.find(selector)
        except Exception as err:
            print(err)

        finally:
            return result
        
    
    @staticmethod
    def parser_table(html: PyQuery) -> list:
        soup = BeautifulSoup(html.html(), 'html.parser')

        # Extract table data
        table_data = []
        thead = soup.find('thead')
        tbody = soup.find('tbody')
        tfoot = soup.find('tfoot')

        trHead = thead.select('tr')

        keys = [[] for a in trHead]

        for i, tr in enumerate(trHead):
            for j, th in enumerate(tr.select('th')):
                rowspan = int(th.get('rowspan', 1))
                colspan = int(th.get('colspan', 1))
                text = ' '.join([a.strip() for a in th.text.strip().split('\n')])
                for h in range(colspan):
                    for d in range(i, rowspan):
                        keys[d].append(text)
                    if i > 0:
                        keys[i].append(text)

        keys.reverse()
        result = []
        for tr in tbody.select('tr'):
            res = []
            for j, key in enumerate(keys):
                r = {}
                for i, td in enumerate(tr.select('td')):
                    text = td.text.strip()
                    if j > 0:
                        if key[i] != keys[j - 1][i]:
                            if key[i] not in r: r[key[i]] = {}
                            r[key[i]][keys[j - 1][i]] = res[j - 1][keys[j - 1][i]]
                        else:
                            r[key[i]] = text
                    else:
                        r[key[i]] = text
                    if (a := td.select_one('a')):
                        r['url'] = a.get('href')
                res.append(r)
            result.append(res[-1])
            
        return result


from .stream import Stream
from .casting import Casting
from .data_manipulation import (
    search_key_v2,
    search_key,
    split_list,
    to_float,
    to_int,
    val,
    vlist_dict
)
from .datetimes import Time
from .directory import Dir
from .downloader import Down
from .endecode import Endecode
from .fileIO import File
from .functions import Funct
from .html_parser import HtmlParser
from .parser import Parser
from .urlParsered import UrlPars


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.6'
DESCRIPTION = 'Utils'

setup(
    name="valkyt",
    version=VERSION,
    author="ryyos (Rio Dwi Saputra)",
    author_email="<riodwi12174@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    include_package_data=True,
    install_requires=['greenstalk', 'kafka-python', 'pika', 'loguru', 'redis', 'boto3', 'python-dateutil', 'playwright', 'dekimashita', 'requests', 'pyquery', 'bs4', 'click', 's3fs', 'unittest'],
    keywords=['valkyt', 'utils'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Operating System :: Microsoft :: Windows",
    ]
)



import json
import os

from loguru import logger

from .connection import ConnectionKafka
from valkyt.utils import Stream, File

class Kafkaa(ConnectionKafka):
    def __init__(self, bootstraps):
        super().__init__(bootstraps)
    
    
    def send(self, data: dict, topic: str, bootstrap: str) -> None:
        logs = self.client.send(topic=topic, value=str.encode(json.dumps(data))).get(timeout=10)
        logger.info(f'SEND KAFKA :: MESSAGE [ {logs} ]')
        Stream.shareKafka(topic)

    @staticmethod
    def local2kafka(source: str, topic: str, bootstrap: str) -> None:
        for root, _, files in os.walk(source.replace('\\', '/')):
            for file in files:
                if file.endswith('json'):
                    file_path = os.path.join(root, file).replace('\\', '/')
                    Stream.shareKafka(topic)
                    data: dict = File.read_json(file_path)
                    Kafkaa.send(data, topic, bootstrap)


import jmespath
import shutil

from .directory import Dir
class Funct:

    @staticmethod
    def copy(source: str, destination: str) -> str:
        Dir.create_dir('/'.join(destination.split('/')[:-1]))
        destination: str = shutil.copy2(source=source, dst=destination)

        return destination
        ...
        
    @staticmethod
    def find(datas: list, value: str, key: str, **kwargs) -> dict:
        if not value:
            return datas
        try:
            for item in datas:
                if int(value) == int(item.get(key)):
                    return item
        except:
            for item in datas:
                if value.lower() in item.get(key).lower():
                    return item